from .MultiPlot import *


class InteractivePlot(MultiPlot):

    FIGSIZE = (15, 15)

    def __init__(self, n_img=5):
        """
        Init the object with the number of images/plots that must contain each Figure.

        :param n_img: Number of images/plots of the figures
        """
        plt.ion()
        super().__init__(n_img)

        self.authomatic = False
        self.interval = 1

    def set_authomatic_loop(self, value, time):
        """
        Specify if the plots has to be updated automatically or manually

        :param value: Activate or deactivate the automatic loop
        :type value: bool

        :param time: Number of seconds to wait bet
        :type time: float
        :return:
        """
        self.authomatic = value
        self.interval = time

    def multi(self, images, cmap="Greys", title=None):
        """
        Plot a multiple image figure in a window.

        :param images: list of dictionaries with structure:
            {
            "img": image to plot,
            "title": name of the plot,
            "colorbar": show colorbar to the image
            "cmap": image matplotlib colormap --> https://matplotlib.org/examples/color/colormaps_reference.html
            }
        :param cmap: Default color map to use in all subplots
        :param title: Main title to show in the figure.
        
        :return:
        """
        super().multi(images, cmap, title)
        if self.authomatic:
            self.fig.canvas.start_event_loop(self.interval)
        else:
            while not self.fig.waitforbuttonpress(0):
                pass



import cv2
from .images import *
from .masks import mask_fill_holes


class LikelihoodGenerator:
    """
    Class to help to generate likelihood masks of regions of
    interest in images.

    Build your likelihood by marking with the mouse the contours of
    the roi region.

    Keyboard commands:
    key a: Submit region selected.
    keb b: discard last mouse click

    Example of use:
    >>> image = "RGB image matrix"
    >>> path_output_file = "Path to save the likelihood"
    >>> LikelihoodGenerator(image).build_your_mask(path_output_file)
    """

    def __init__(self, image):
        """
        Init an object to build the interest region of an image
        :param image: RGB image
        """
        self.image = image.copy()

        self.in_points = []  # etiquetes de les finger_regions seleccionades

    def __fill_event(self, event, x, y, flags, param):
        """
        Event to execute when click onto a region
        :param event:
        :param x:
        :param y:
        :param flags:
        :param param:
        :return:
        """
        if event == cv2.EVENT_LBUTTONDOWN:
            point = (x, y)
            print("event at position: " + str(point))

            cv2.circle(self.image, point, 5, (0, 0, 0), -1)

            if len(self.in_points) != 0:
                cv2.line(self.image, self.in_points[-1], point, (0, 0, 0))

            self.in_points.append(point)

            cv2.destroyWindow("image")

    def build_your_mask(self, file=None):
        """
        Call this function to init the process of finger_regions selection.
        In the process:
        Press A (shift + a) to end selection
        Press b to cancel the last selection
        :return:
        """
        key = 0
        while "a" != chr(key & 255):
            cv2.namedWindow("image", flags=cv2.WINDOW_NORMAL)
            cv2.setMouseCallback("image", self.__fill_event)
            cv2.resizeWindow('image', 600, 600)

            # display the image and wait for a keypress
            cv2.imshow("image", self.image)
            key = cv2.waitKey(-1)
            print("pressed: " + str(chr(key & 255)))

            if "b" == chr(key & 255):
                value = self.in_points.pop()
                cv2.line(self.image, self.in_points[-1], value, (255, 255, 255))

        # close all open windows
        cv2.destroyAllWindows()

        mask = np.zeros(self.image.shape[0:2], dtype=np.uint8)
        p1 = self.in_points[0]
        for idx in range(1, len(self.in_points)):
            p2 = self.in_points[idx]
            cv2.line(mask, p1, p2, 255, 1)
            p1 = p2

        mask *= 255
        mask = mask_fill_holes(mask)
        if file is not None:
            print("Writting file to: " + str(file))
            # TODO: check if folder exists, if not create dir
            cv2.imwrite(file, binary2RGB(mask))

        return mask



import cv2
import numpy as np
import os
from copy import copy

import sklearn.metrics as metrics


def mask_evaluation(mask, likelihood):
    """
    Get the evaluation metrics of a given mask and his likelihood
    :param mask: Binary image => Mask to evaluate
    :param likelihood: Binary image => Correct mask
    :raise Incorrect likelihood
    :return: dict{
        "FN"
        "TP"
        "FP"
        "Recall"
        "Precision"
        "F1"
        "cohen_kappa"
        "accuracy"
    }

    """
    stats = {}

    likelihood = np.copy(likelihood).astype(np.uint8)
    P = np.count_nonzero(likelihood)  # the number of real positive cases in the data
    invert_likelihood = np.bitwise_not(likelihood)
    N = np.count_nonzero(invert_likelihood)  # the number of real negative cases in the data

    if N == 0 or P == 0:
        raise Exception("Incorrect likelihood")

    # pixels identificats com a negatius que haurien de ser positius
    fn = np.count_nonzero(np.bitwise_and(np.bitwise_not(mask), likelihood))

    # pixels identificats com a positius que haurien de ser positius
    tp = np.count_nonzero(np.bitwise_and(mask, likelihood))

    # pixels identificats com a positius que haurien de ser negatius
    fp = np.count_nonzero(np.bitwise_and(mask, np.bitwise_not(likelihood)))

    tn = N - fp

    stats["FNR"] = fn / P
    stats["FPR"] = fp / N

    stats["TPR"] = tp / P
    stats["TNR"] = tn / N

    if stats["TPR"] != 0 or stats["FPR"] != 0:
        stats["Precision"] = tp / (tp + fp)
    else:
        stats["Precision"] = 0

    if stats["Precision"] != 0 or stats["TPR"] != 0:
        stats["F1"] = (2 * stats["Precision"] * stats["TPR"]) / (stats["Precision"] + stats["TPR"])
        stats["accuracy"] = (tp + tn) / (P + N)
        stats["cohen_kappa"] = metrics.cohen_kappa_score(likelihood.flatten(), mask.flatten())

    return stats


def mask_sklearn_evaluation(mask, likelihood, pos_label=255):
    """
    Same than mask_evaluation, but using sklearn library for compute values
    :param mask:
    :param likelihood:
    :return:
    """
    y_true = likelihood.flatten()
    y_pred = mask.flatten()

    tn, fp, fn, tp = metrics.confusion_matrix(y_true, y_pred).ravel()

    N = tn + fp
    P = fn + tp

    prfs = metrics.precision_recall_fscore_support(y_true, y_pred, pos_label=pos_label, average="binary")

    stats = {
        "Precision": tp / (tp + fp),
        "TPR": prfs[1],
        "TNR": tn / N,
        "FPR": fp / N,
        "FNR": fn / P,
        "Fbeta": prfs[2],
        "cohen_kappa": metrics.cohen_kappa_score(y_true, y_pred),
        "accuracy": (tp + tn) / (P + N),
        "r2": metrics.r2_score(y_true, y_pred),
        "F1": metrics.f1_score(y_true, y_pred, pos_label=pos_label)
    }

    return stats


def masks_coincidence(mask1, mask2, priority="big_mask"):
    """
    Get the porcentage of coincidence between two masks of the same shape
    :param priority:
    :param mask1:
    :param mask2:
    :return:
    """
    mask1 = mask1.astype(np.uint8)
    mask2 = mask2.astype(np.uint8)

    res = np.bitwise_and(mask1, mask2)

    # number of coincident pixels
    equals = np.count_nonzero(res)
    n_pix1 = np.count_nonzero(mask1)
    n_pix2 = np.count_nonzero(mask2)

    if priority == "small_mask":
        max_pix = np.min([n_pix1, n_pix2])
    else:
        max_pix = np.max([n_pix1, n_pix2])

    return equals / max_pix


def mask_onto_mask(mask1, mask2, perc=0.9):
    """
    Given two masks of the same shape, check if them are one onto the other.

    :param mask1:
    :param mask2:
    :param perc: percentage of coincidence pixels that must have the two masks
                    to consider that them are one onto the other.
    :return: bool
    """
    res = np.bitwise_and(mask1.astype(np.bool), mask2.astype(np.bool))

    # number of coincident pixels
    equals = np.count_nonzero(res)

    n_pix1 = np.count_nonzero(mask1)
    n_pix2 = np.count_nonzero(mask2)

    return equals > (n_pix1 * perc) or equals > (n_pix2 * perc)


def mask_2RGB(mask):
    """
    Convert a binary image to RGB, black & white image
    :param mask: binary image
    :return: binary image coverted to RGB, true values (or 1) to BLACK and other to WHITE
    """
    mask = mask.copy()
    img_result = mask.astype(np.uint8)
    mask = np.zeros(img_result.shape).astype(np.uint8)

    res = np.zeros((img_result.shape[0], img_result.shape[1], 3), dtype=np.uint8)
    res[np.equal(img_result, mask)] = (255, 255, 255)
    return res


def mask_fill_holes(mask):
    """
    Fill all the empty pixels overwhelmed by true pixels
    :param mask: 0, 255 mask
    :return: O values inside 255 values filled with 255
    """
    # Threshold.
    # Set values equal to or above 220 to 0.
    # Set values below 220 to 255.
    mask = mask.copy()
    # mask = cv2.bitwise_not(mask)
    if mask.max() == 1:
        im_th = mask * 255

    else:
        mask = cv2.bitwise_not(mask)
        th, im_th = cv2.threshold(mask, 220, 255, cv2.THRESH_BINARY_INV)

    # Copy the thresholded image.
    im_floodfill = im_th.copy()

    # Mask used to flood filling.
    # Notice the size needs to be 2 pixels than the image.
    h, w = im_th.shape[:2]
    mask = np.zeros((h + 2, w + 2), np.uint8)

    # Floodfill from point (0, 0)
    cv2.floodFill(im_floodfill, mask, (0, 0), 255)

    # Invert floodfilled image
    im_floodfill_inv = cv2.bitwise_not(im_floodfill)

    # Combine the two images to get the foreground.
    im_out = im_th | im_floodfill_inv

    if mask.max() == 1:
        im_out = im_out / 255

    return im_out.astype(np.uint8)


def mask_from_RGB_file(file_mask):
    """
    Build a mask from a BLACK & white RGB image from file
    :param file_mask: path of the file to build
    :return: uint8 image
    """
    if not os.path.exists(file_mask):
        return None

    mask = cv2.imread(file_mask)
    mask = cv2.cvtColor(mask, cv2.COLOR_RGBA2GRAY).astype(np.uint8)
    img_bool = mask.astype(np.bool)
    mask[np.logical_not(img_bool)] = 1
    mask[img_bool] = 0
    return mask


def mask_bounding_circle(mask):
    """
    Get the minium enclosing circle of a given mask
    :param mask:
    :return: (x, y), r => integers
    """
    _, th = cv2.threshold(mask.copy(), 1, 255, cv2.THRESH_BINARY_INV)
    _, contours, _ = cv2.findContours(th.copy(), cv2.RETR_LIST, cv2.CHAIN_APPROX_NONE)
    cnt = contours[0]

    (x, y), radius = cv2.minEnclosingCircle(cnt)
    center = (int(x), int(y))
    radius = int(radius)
    return center, radius


def mask_delete_contour_in(mask, region):  # TODO: comment function
    """

    :param mask:
    :param region:
    :return:
    """
    mask = mask.astype(np.uint8)

    _, contours, _ = cv2.findContours(mask.copy(), cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)

    for contour in contours:

        area = cv2.contourArea(contour)
        if area > 100:
            M = cv2.moments(contour)
            cx = int(M['m10'] / M['m00'])
            cy = int(M['m01'] / M['m00'])

            if cy < region:
                cv2.drawContours(mask, [contour], 0, 0, -1)

    return mask


def mask_build_circular(image, circle):
    """
    Build a circular mask onto the image
    :param image: 3 channels image
    :param circle: (int,int, int)
        tuple with the circle to find, two first values are circle coordinates (x, y), the thirth is the radius
    :return:
    """
    center = (circle[0], circle[1])
    radius = circle[2]

    mask = np.zeros(image.shape, dtype=np.uint8)
    cv2.circle(mask, center, radius, (255, 255, 255), -1)

    image_masked = image & mask

    return image_masked


def mask_build_circular_boolean(image, circle):
    """
    build a circular binary onto the image
    :param image: image
    :param circle: (int,int, int)
        tuple with the circle to find, two first values are circle coordinates (x, y), the thirth is the radius
    :return:
    """
    center = (circle[0], circle[1])
    radius = circle[2]

    mask = np.zeros(image.shape[:2], dtype=np.uint8)
    cv2.circle(mask, center, radius, (255), -1)

    return np.asarray(mask, dtype=np.bool)


def mask_biggest_connected_component(mask):
    """
    Get the biggest connected component of a mask
    :param mask:
    :return:
    """
    mask = mask.copy().astype(np.uint8)
    mask[mask != 0] = np.iinfo(np.uint8).max

    # Find the largest contour and extract it
    im, contours, hierarchy = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
    if len(contours) == 0:
        return mask

    biggest_cnt = max(contours, key=cv2.contourArea)

    # Create a mask from the largest contour
    mask = np.zeros_like(mask)
    cv2.fillPoly(mask, [biggest_cnt], 1)

    return mask


def mask_every_separated(masks):
    """
    Get an image of every component in the mask with different color
    :param masks:
    :return:
    """
    masks = masks.copy().astype(np.uint8)
    masks[masks != 0] = np.iinfo(np.uint8).max
    # Find the largest contour and extract it
    _, contours, _ = cv2.findContours(masks, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)

    multi_mask = np.zeros_like(masks)
    color = 1
    for cnt in contours:
        cv2.fillPoly(multi_mask, [cnt], color)
        color = color + 1

    return multi_mask


def mask_2BOOL(mask):
    """
    Convert mask to false true value

    TODO: check if numpy astype(bool) make the same
    :param mask:
    :return:
    """

    img_cp = copy.copy(mask)

    img_cp[mask == 0] = False
    img_cp[mask != 0] = True

    return img_cp


def RGB_2MASK(img, threshold):
    """
    Create a binary image from multichannel image. First does the thresholding and then use the and operation

    :param img:
    :param threshold:
    :return:
    """

    mask = copy(img)
    mask[mask < threshold] = 0
    mask[mask >= threshold] = 1

    r, g, b = cv2.split(mask)
    mask = cv2.bitwise_and(r, g)
    mask = cv2.bitwise_and(mask, b)
    mask = cv2.bitwise_not(mask) + 2

    return mask



import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable


class MultiPlot:
    """
    Class to help to plot a set of images or plots in a unique figure.

    """

    FIGSIZE = (15, 15)

    def __init__(self, n_img=5):
        """
        Init the object with the number of images/plots that must contain each Figure.

        :param n_img: Number of images/plots of the figures
        """
        self.colorbars = []
        self.n_img = n_img
        if n_img == 1:
            self.fig = plt.Figure(self.FIGSIZE)
            self.axes = [plt.axes()]
        elif n_img > 1:
            self.__create_figure()
        else:
            raise Exception("Bad number of images")

    def __create_figure(self, keep_unused_axes=False):
        """
        Internal function that build the Figure that are going to contain
        all the plots

        :param keep_unused_axes: Specify if the axes of the subplots must be keep
        :type keep_unused_axes: bool

        :return:
        """
        nrows = 2
        if (self.n_img % 2) == 0:
            columns = int(self.n_img / nrows)
        else:
            columns = int((self.n_img / nrows) + 1)
        self.fig, self._axes = plt.subplots(nrows=nrows, ncols=columns, figsize=self.FIGSIZE)
        self.axes = self._axes.flatten()[:self.n_img]
        self.unused_axes = self._axes.flatten()[self.n_img:]
        if not keep_unused_axes:
            for uaxes in self.unused_axes:
                uaxes.remove()
            self.unused_axes = []

        self.fig.canvas.set_window_title('Interaction figure')

    def _multiplot(self, images, cmap='Greys', title=None):
        """
        Help to plot a multiple image figure
        :param images: list of dictionaries with structure:
            {
            "img": image to plot,
            "title": name of the plot,
            "colorbar": show colorbar to the image
            "cmap": image matplotlib colormap --> https://matplotlib.org/examples/color/colormaps_reference.html
            }
        :param cmap: Default color map to use in all subplots
        :param title: Main title to show in the figure.

        :return:
        """
        if title is not None:
            self.fig.suptitle(title, fontsize=16)

        for cb in self.colorbars:
            cb.remove()
        self.colorbars = []
        for i, img in enumerate(images):
            ax = self.axes[i]

            image = img['img']
            title = img['title']

            ax.clear()
            ax.set_title(title)
            ax.axis('off')
            if 'cmap' in img:
                imshow = ax.imshow(image, cmap=img['cmap'])

            else:
                imshow = ax.imshow(image, cmap=cmap)

            if 'colorbar' in img:
                im = ax.images  # this is a list of all images that have been plotted
                cb = im[-1].colorbar
                if cb is not None:
                    cb.remove()
                divider = make_axes_locatable(ax)
                cax = divider.append_axes('right', size='5%', pad=0.05)
                cb = self.fig.colorbar(imshow, cax=cax, orientation='vertical')
                self.colorbars.append(cb)
                
    def multi(self, images, cmap="Greys", title=None):
        """
        Plot a multiple image figure in a window.

        :param images: list of dictionaries with structure:
            {
            "img": image to plot,
            "title": name of the plot,
            "colorbar": show colorbar to the image
            "cmap": image matplotlib colormap --> https://matplotlib.org/examples/color/colormaps_reference.html
            }
        :param cmap: Default color map to use in all subplots
        :param title: Main title to show in the figure.

        :return:
        """
        if len(images) > len(self.axes):
            self.__create_figure()

        self._multiplot(images, cmap, title)

    def save_multiplot(self, filename, images, cmap="Greys"):
        """
        Plot a multiple image figure in a file.

        :param filename: String|None
                            Path file Where save the figure,
                            None to plot in a window

        :param images: list of dictionaries with structure:
            {
            "img": image to plot,
            "title": name of the plot,
            "colorbar": show colorbar to the image
            "cmap": image matplotlib colormap --> https://matplotlib.org/examples/color/colormaps_reference.html
            }

        :param cmap: Default color map to use in all subplots

        :return:
        """
        self._multiplot(images, cmap)
        plt.savefig(filename, bbox_inches='tight')


import cv2


def images2video(images, video_name, fps):
    """
    Write a sequence of images as a video

    :param images: List of images
    :param video_name: String Name of the result video file
    :param fps: int Number of frames per second
    :return:
    """
    frame = cv2.imread(images[0])
    height, width, layers = frame.shape

    video = cv2.VideoWriter(video_name, cv2.VideoWriter_fourcc('M', 'J', 'P', 'G'), fps=fps, frameSize=(width, height))
    for image in images:
        video.write(cv2.imread(image))

    # cv2.destroyAllWindows()
    video.release()



import os
from itertools import combinations


def potencia(input, min_n_items=1, max_n_items=None):
    """Calcula y devuelve el conjunto potencia del
       conjunto input.
    """
    if max_n_items is None:
        max_n_items = len(input) + 1

    out = []
    for i in range(min_n_items, max_n_items):

        out += list(combinations(input, i))

    return out


"""
    FILE SYSTEM FUNCTIONS
"""


def check_dir(path):
    """
    Check if the specified path exists into file system. If not exists, it is created
    :param path: String
    :return:
    """
    if not os.path.exists(path):
        os.makedirs(path)



import cv2
import numpy as np


def binary2RGB(image):
    """
    Convert a binary image to RGB, black & white image
    O values to BLACK others to white
    :param image: binary image
    :return: binary image coverted to RGB, true values (or 1) to BLACK and other to WHITE
    """
    img = image.astype(np.uint8)
    mask = np.zeros(img.shape, dtype=np.uint8)

    res = np.zeros((img.shape[0], img.shape[1], 3), dtype=np.uint8)
    res[np.equal(img, mask)] = (255, 255, 255)
    return res


def grays2binary(image_grays):
    """
    Convert a grayscale image to a binary one
    :param image_grays: one chanel image
    :return:
    """
    results = np.zeros(image_grays.shape)
    results[np.not_equal(image_grays, results)] = 1
    return results


def reduce_image(img, size):
    """
    Resize the image without deformation
    :param img:
    :param size: Tuple (witdh, height)
    :return:
    """

    witdh, height = size
    if len(img.shape) > 2:
        shape = (witdh, height, img.shape[2])
    else:
        shape = (witdh, height)
    back = np.full(shape, 255)
    shap = img.shape

    if shap[0] > shap[1]:
        r_height = witdh
        r_width = int((witdh / shap[0]) * shap[1])
    else:
        r_height = int((height / shap[1]) * shap[0])
        r_width = height

    aux = cv2.resize(img, (r_width, r_height))

    w_off = int((back.shape[0] - aux.shape[0]) / 2)
    h_off = int((back.shape[1] - aux.shape[1]) / 2)

    if len(img.shape) > 2:
        back[w_off:aux.shape[0] + w_off, h_off:aux.shape[1] + h_off, :] = aux
    else:
        back[w_off:aux.shape[0] + w_off, h_off:aux.shape[1] + h_off] = aux

    return back




from .plots.InteractivePlot import *
from .plots.plots import *
from .plots.video import *
from .LikelihoodGenerator import *
from .decorators import *
from .functions import *
from .images import *
from .masks import *



from setuptools import setup

setup(name='ovnImage',
      version='0.2.2',
      description='Useful helper functions of image processing and machine learning',
      url='',
      author="Miquel Miró Nicolau, Bernat Galmés Rubert, Dr. Gabriel Moyà Alcover",
      author_email='miquelca32@gmail.com, bernat_galmes@hotmail.com, gabriel_moya@uib.es',
      license='MIT',
      packages=['ovnImage', 'ovnImage.plots'],
      install_requires=[
          'scikit-learn',
          'matplotlib',
          'numpy',
          'seaborn',
          'scikit-image',
      ],
      zip_safe=False)



"""
Useful functions to easily create new plots
"""
import cv2
import numpy as np
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.pyplot as plt
import seaborn as sns

from ..images import binary2RGB

from skimage.segmentation import find_boundaries
from mpl_toolkits.axes_grid1 import make_axes_locatable


def multiplot(images, filename=None, nrows=2, cmap='Greys'):
    """
    Help to plot a multiple image figure
    :param images: list of dictionaries with structure:
        {
        "img": image to plot,
        "title": name of the plot
        }
    :param filename: String|None
                        Path file Where save the figure,
                        None to plot in a window
    :return:
    """
    plt.clf()
    n_img = len(images)

    if (n_img % 2) == 0:
        columns = int(n_img / nrows)
    else:
        columns = int((n_img / nrows) + 1)

    for i, img in enumerate(images):
        image = img['img']
        title = img['title']

        ax = plt.subplot(nrows, columns, i + 1)
        ax.set_title(title)
        ax.axis('off')
        if 'cmap' in img:
            imshow = ax.imshow(image, cmap=img['cmap'])

        else:
            imshow = ax.imshow(image, cmap=cmap)

        if 'colorbar' in img:
            divider = make_axes_locatable(ax)
            cax = divider.append_axes('right', size='5%', pad=0.05)
            plt.colorbar(imshow, cax=cax, orientation='vertical')

    if filename:
        plt.savefig(filename, bbox_inches='tight')
    else:
        plt.show()


def plots_correlation_matrix(df, labels=None, absolute=False, method='pearson'):
    """
    Get correlation matrix of dataframe columns
    :param df: pandas dataframe
    :param labels:  String[]
                    labels of each dataframe column
    :return: matplotlib Axes
            Axes object with the heatmap.
    """
    corr = df.corr(method=method)

    if labels is None:
        labels = corr.columns.values

    if absolute:
        corr = np.absolute(corr)
    corrmap = sns.heatmap(corr,
                          xticklabels=labels,
                          yticklabels=labels,
                          center=0)
    corrmap.set_yticklabels(corrmap.get_yticklabels(), rotation=45, fontsize=8)
    corrmap.set_xticklabels(corrmap.get_xticklabels(), rotation=90, fontsize=8)
    return corrmap


def plots_segmentation(img, labels):
    """
    Plot the results of an image segmentation
    Example of use:
    >>> from skimage.segmentation import slic
    >>> img = cv2.imread("path/to/image")
    >>> img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    >>> segments = slic(img, n_segments=20, compactness=10, max_iter=100, sigma=2.3)
    >>> plots_segmentation(img, segments)
    :param img: RGB image
    :param labels: 2D or 3D array
                    Integer mask indicating segment labels.
    :return:
    """
    img_res = img.copy()
    boundaries = find_boundaries(labels).astype(np.uint)
    img_res = cv2.addWeighted(img_res, 0.7, binary2RGB(boundaries), 0.3, 0)
    plt.imshow(img_res)
    plt.show()


def plots_raw_data(df, columns, colors="b"):
    """
    Plot raw data of the given dataframe
    :param df: DataFrame Wich contains all the data
    :param columns: List strings name of the columns to plot
    :param colors: color, sequence, or sequence of color, optional, default: ‘b’
                    c can be a single color format string, or a sequence of color specifications of length N,
                    or a sequence of N numbers to be mapped to colors using the cmap and norm specified
                    via kwargs (see below).
                    Note that c should not be a single numeric RGB or RGBA sequence because that is indistinguishable
                    from an array of values to be colormapped. c can be a 2-D array in which the rows are RGB or RGBA,
                    however, including the case of a single row to specify the same color for all points.
    :return:
    """
    if len(columns) == 3:
        projection = "3d"
    else:
        projection = "rectilinear"

    fig = plt.figure()
    ax = fig.add_subplot(111, projection=projection)
    if projection == "3d":
        ax.scatter(df[columns[0]], df[columns[1]], df[columns[2]], c=colors, marker='o')
        ax.set_xlabel(columns[0])
        ax.set_ylabel(columns[1])
        ax.set_zlabel(columns[2])
    else:
        ax.scatter(df[columns[0]], df[columns[1]], c=colors, marker='o')
        ax.set_xlabel(columns[0])
        ax.set_ylabel(columns[1])
    plt.show()


# TODO: not tryed
def biplot(pca, dat):
    """
    Plot a data biplot on the screen
    :param dat: DataFrame data to plot
    :return:
    """
    print("computing biplot ...")
    # 0,1 denote PC1 and PC2; change values for other PCs
    xvector = pca.components_[0]  # see 'prcomp(my_data)$rotation' in R
    yvector = pca.components_[1]

    xs = pca.transform(dat)[:, 0]  # see 'prcomp(my_data)$x' in R
    ys = pca.transform(dat)[:, 1]

    # visualize projections

    # Note: scale values for arrows and text are a bit inelegant as of now,
    #       so feel free to play around with them

    # plt.figure(1)
    for i in range(len(xs)):
        # circles project documents (ie rows from csv) as points onto PC axes
        plt.plot(xs[i], ys[i], 'b,')
        # plt.text(xs[i] * 1.2, ys[i] * 1.2, list(dat.index)[i], color='b')

    for i in range(len(xvector)):
        # arrows project features (ie columns from csv) as vectors onto PC axes
        plt.arrow(0, 0, xvector[i] * max(xs), yvector[i] * max(ys),
                  color='r', width=0.0005, head_width=0.0025)
        plt.text(xvector[i] * max(xs) * 1.2, yvector[i] * max(ys) * 1.2,
                 list(dat.columns.values)[i], color='r')

    plt.show()
